package sw.programme.rfid.uhf.demo;

import android.content.Context;
import android.util.Log;

import com.cipherlab.rfid.DeviceResponse;
import com.cipherlab.rfid.InventoryType;
import com.cipherlab.rfid.LockTarget;
import com.cipherlab.rfid.RFIDMemoryBank;
import com.cipherlab.rfid.RFIDMode;
import com.cipherlab.rfidapi.RfidManager;


public class AppHelper {
    private static Context mContext;
    private static RfidManager mManager = null;
    private static volatile AppHelper instance = null;
    public static String TAG = "demo";

    public static synchronized AppHelper getInstance() {
        if (instance == null) {
            instance = new AppHelper();
        }
        return instance;
    }

    public static boolean Initial(Context context) {
        mContext = context;
        mManager = RfidManager.InitInstance(context);
        return true;
    }

    public void Release() {
        if (mManager != null) {
            mManager.Release();
        }
    }

    public Context getContext() {
        return mContext;
    }

    public boolean GetConnectionStatus() {
        if (mManager == null)
            return false;
        else
            return mManager.GetConnectionStatus();
    }

    public void ScanTrigger(boolean flag) {
        if (mManager != null) {
            int ret = mManager.SoftScanTrigger(flag);
            if (ret != 0) {
                Log.i(TAG, "SoftScanTrigger = " + mManager.GetLastError());
            } else {
                Log.i(TAG, "SoftScanTrigger = " + flag);
            }
        }
    }

    public void SetRFIDMode(RFIDMode mode) {
        if (mManager != null) {
            int ret = mManager.SetRFIDMode(mode);
            if (ret != 0) {
                Log.i(TAG, "SetRFIDMode = " + mManager.GetLastError());
            } else {
                Log.i(TAG, "SetRFIDMode = " + mode);
            }
        }
    }

    public RFIDMode GetRFIDMode() {
        if (mManager != null) {
            RFIDMode mode = mManager.GetRFIDMode();
            Log.i(TAG, "SetRFIDMode=" + mode);
            return mode;
        }
        return RFIDMode.Err;
    }

    public void SetHardwareTriggerStatus(boolean enable) {
        if (mManager != null) {
            int ret = mManager.EnableDeviceTrigger(enable);
            if (ret != 0) {
                Log.i(TAG, "EnableDeviceTrigger = " + mManager.GetLastError());
            } else {
                Log.i(TAG, "EnableDeviceTrigger = " + enable);
            }
        }
    }

    public boolean GetHardwareTriggerStatus() {
        int Status = mManager.DeviceTriggerStatus();
        if (Status == -1) {
            Log.i(TAG, "DeviceTriggerStatus = " + mManager.GetLastError());
        } else {
            Log.i(TAG, "DeviceTriggerStatus = " + Status);
        }

        if (Status == 1)
            return true;
        else
            return false;
    }

    public void Inventory() {
        if (mManager != null) {
            int ret = mManager.RFIDDirectStartInventoryRound(InventoryType.EPC, 1);
            if (ret != 0) {
                Log.i(TAG, "RFIDDirectStartInventoryRound = " + mManager.GetLastError());
            } else {
                Log.i(TAG, "RFIDDirectStartInventoryRound");
            }
        }
    }

    public boolean ReadTag(byte[] password, byte[] epc, RFIDMemoryBank bank, int start, int length) {

        if (mManager != null) {
            int ret = mManager.RFIDDirectReadTagByEPC(password, epc, bank, start, length, 3);
            if (ret != 0) {
                Log.i(TAG, "RFIDDirectReadTagByEPC = " + mManager.GetLastError());
            } else {
                Log.i(TAG, String.format("RFIDDirectReadTagByEPC bank=%s start=%d length=%d",
                        bank.toString(), start, length));
            }
        }
        return true;
    }

    public boolean WriteTag(byte[] password, byte[] epc, RFIDMemoryBank bank, int start, byte[] data) {
        if (mManager != null) {
            DeviceResponse ret = mManager.RFIDDirectWriteTagByEPC(password, epc, bank, start, 3, data);
            if (ret != DeviceResponse.OperationSuccess) {
                Log.i(TAG, "RFIDDirectWriteTagByEPC = " + mManager.GetLastError());
            } else {
                Log.i(TAG, String.format("RFIDDirectWriteTagByEPC bank=%s start=%d data=%s",
                         bank.toString(), start, data));
            }
        }
        return true;
    }

    public boolean LockTag(byte[] password, byte[] epc, LockTarget target) {
        if (mManager != null) {
            DeviceResponse ret = mManager.RFIDDirectLockTag(password, epc, target);
            if (ret != DeviceResponse.OperationSuccess) {
                Log.i(TAG, "RFIDDirectLockTag = " + mManager.GetLastError());
            } else {
                Log.i(TAG, String.format("RFIDDirectLockTag target=%s", target.toString()));
            }
        }
        return true;
    }

    public boolean UnlockTag(byte[] password, byte[] epc, LockTarget target) {
        if (mManager != null) {
            DeviceResponse ret = mManager.RFIDDirectUnlockTag(password, epc, target);
            if (ret != DeviceResponse.OperationSuccess) {
                Log.i(TAG, "RFIDDirectUnlockTag = " + mManager.GetLastError());
            } else {
                Log.i(TAG, String.format("RFIDDirectUnlockTag target=%s", target.toString()));
            }
        }
        return true;
    }
}
