package sw.programme.rfid.uhf.demo;

import android.content.Context;
import android.content.Intent;
import android.content.res.Resources;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.InputFilter;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;

import com.cipherlab.rfid.GeneralString;
import com.cipherlab.rfid.LockTarget;


public class LockFragment extends Fragment {

    private EditText mEPC;
    private EditText mPassword;
    private Spinner mTarget;

    private Button mLockBtn;
    private Button mUnlockBtn;

    private final AppHelper helper = AppHelper.getInstance();

    public LockFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        final View rootView = inflater.inflate(R.layout.fragment_lock, container, false);

        mEPC = rootView.findViewById(R.id.edt_lock_epc);
        mPassword = rootView.findViewById(R.id.edt_lock_password);
        InputFilter[] FilterArray1 = new InputFilter[1];
        FilterArray1[0] = new InputFilter.LengthFilter(8);
        mPassword.setFilters(FilterArray1);

        Resources res = getResources();
        mTarget = rootView.findViewById(R.id.select_lock_bank);
        ArrayAdapter<String> adapter1 = new ArrayAdapter<String>(rootView.getContext(),
                R.layout.dropdown_item, res.getStringArray(R.array.target_option));
        mTarget.setAdapter(adapter1);
        mTarget.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

            @Override
            public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id) {


            }

            @Override
            public void onNothingSelected(AdapterView<?> parentView) {

            }
        });

        mLockBtn = rootView.findViewById(R.id.btn_lock);
        mLockBtn.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                byte[] EPCByteArray = GetEpcData();
                if (EPCByteArray == null)
                    return;

                byte[] PasswordByteArray = GetPassWordData();
                if (PasswordByteArray == null)
                    return;

                switch (mTarget.getSelectedItemPosition()) {
                    case 0:
                        helper.LockTag(PasswordByteArray, EPCByteArray, LockTarget.KillPassword);
                        break;
                    case 1:
                        helper.LockTag(PasswordByteArray, EPCByteArray, LockTarget.AccessPassword);
                        break;
                    case 2:
                        helper.LockTag(PasswordByteArray, EPCByteArray, LockTarget.EPCBank);
                        break;
                    case 3:
                        helper.LockTag(PasswordByteArray, EPCByteArray, LockTarget.TIDBank);
                        break;
                    case 4:
                        helper.LockTag(PasswordByteArray, EPCByteArray, LockTarget.UserBank);
                        break;
                }
            }
        });

        mUnlockBtn = rootView.findViewById(R.id.btn_unlock);
        mUnlockBtn.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                byte[] EPCByteArray = GetEpcData();
                if (EPCByteArray == null)
                    return;

                byte[] PasswordByteArray = GetPassWordData();
                if (PasswordByteArray == null)
                    return;

                switch (mTarget.getSelectedItemPosition()) {
                    case 0:
                        helper.UnlockTag(PasswordByteArray, EPCByteArray, LockTarget.KillPassword);
                        break;
                    case 1:
                        helper.UnlockTag(PasswordByteArray, EPCByteArray, LockTarget.AccessPassword);
                        break;
                    case 2:
                        helper.UnlockTag(PasswordByteArray, EPCByteArray, LockTarget.EPCBank);
                        break;
                    case 3:
                        helper.UnlockTag(PasswordByteArray, EPCByteArray, LockTarget.TIDBank);
                        break;
                    case 4:
                        helper.UnlockTag(PasswordByteArray, EPCByteArray, LockTarget.UserBank);
                        break;
                }
            }
        });

        if (helper.GetConnectionStatus())
            EnableControl(true);
        else
            EnableControl(false);
        return rootView;
    }

    private byte[] GetEpcData() {
        String epc = mEPC.getText().toString();
        if (StringEX.IsNullOrEmpty(epc)) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_EpcEmpty);
            return null;
        }

        if(epc.length() % 2 == 1)
        {
            StringEX.ShowMessage(getActivity(), R.string.MSG_BinaryError);
            return null;
        }

        byte[] ByteArray;
        try {
            Log.i(AppHelper.TAG, "EPC = " + epc);
            ByteArray = UtilityHelp.StringToByteArray(epc);
        } catch (Exception e) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_IncorrectEPC);
            return null;
        }

        return ByteArray;
    }

    private byte[] GetPassWordData() {
        String password = mPassword.getText().toString();
        if (StringEX.IsNullOrEmpty(password)) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_PasswordEmpty);
            return null;
        }

        if (password.length() != 8) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_PasswordLengthError);
            return null;
        }

        byte[] PasswordByteArray;
        try {
            Log.i(AppHelper.TAG, "Password = " + password);
            PasswordByteArray = UtilityHelp.StringToByteArray(password);
        } catch (Exception e) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_IncorrectPassword);
            return null;
        }
        return PasswordByteArray;
    }

    public void ResetField() {
        mEPC.setText("");
        mPassword.setText("");
        mTarget.setSelection(0);
    }

    public void EnableControl(boolean flag) {
        if (mLockBtn != null)
            mLockBtn.setEnabled(flag);
        if (mUnlockBtn != null)
            mUnlockBtn.setEnabled(flag);
    }

    public void SetEPC(String epc) {
        mEPC.setText(epc);
    }

    public void Process(Context context, Intent intent) {
        int type = intent.getIntExtra(GeneralString.EXTRA_DATA_TYPE, -1);
        Log.i(AppHelper.TAG, "type = " + type);
        int response = intent.getIntExtra(GeneralString.EXTRA_RESPONSE, -1);

        switch (response) {
            case 0: /* OPERATION_SUCCESS */
                Log.i(AppHelper.TAG, "OPERATION_SUCCESS");
                break;
            case 1: /* OPERATION_FINISH */
                Log.i(AppHelper.TAG, "OPERATION_FINISH");
                StringEX.ShowMessage(context, R.string.MSG_OperationSuccess);
                break;
            case 2: /* OPERATION_FAIL */
            case 7:
                Log.i(AppHelper.TAG, "OPERATION_FAIL");
                StringEX.ShowMessage(context, R.string.MSG_OperationFail);
                break;
            case 3: /* TAG_LOCK */
                Log.i(AppHelper.TAG, "TAG_LOCK");
                StringEX.ShowMessage(context, R.string.MSG_TagLocked);
                break;
            case 6:
                Log.i(AppHelper.TAG, "Access Password Fail");
                StringEX.ShowMessage(context, R.string.MSG_AccessPasswordFail);
                break;
            case 251: /* DEVICE_BUSY */
                Log.i(AppHelper.TAG, "DEVICE_BUSY");
                StringEX.ShowMessage(context, R.string.MSG_OperationFail);
                break;
            default:
                Log.i(AppHelper.TAG, "response = " + response);
                break;
        }
    }
}
