package sw.programme.rfid.uhf.demo;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.CountDownTimer;
import android.support.design.widget.TabLayout;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AppCompatActivity;
import android.support.v4.view.ViewPager.OnPageChangeListener;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import com.cipherlab.rfid.GeneralString;

import java.util.ArrayList;
import java.util.List;

import sw.programme.rfid.uhf.permission.AppPermissionHelper;

public class MainActivity extends AppCompatActivity {

    private enum StateMachine {
        State_Idle,
        State_Ready
    }

    private TabLayout tabLayout;
    private ViewPager viewPager;
    private ViewPagerAdapter adapter;

    private final AppHelper helper = AppHelper.getInstance();
    private static ProgressDialog PDialog = null;
    private StateMachine mCurrent = StateMachine.State_Idle;
    private static int mPos;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (!AppPermissionHelper.allowPermission(this)) {
            return;
        }

        setContentView(R.layout.activity_main);

        mCurrent = StateMachine.State_Idle;
        helper.Initial(this);

        mPos = 0;
        viewPager = findViewById(R.id.viewpager);
        setupViewPager(viewPager);

        tabLayout = findViewById(R.id.tabs);
        tabLayout.setupWithViewPager(viewPager);

        if (mCurrent == StateMachine.State_Idle) {
            PDialog = null;
            PDialog = ProgressDialog.show(this,
                    getString(R.string.app_name),
                    getString(R.string.MSG_ServiceWait),
                    true);
            final CountDownTimer timer = new CountDownTimer(6000, 1500) {
                @Override
                public void onFinish() {

                    PDialog.dismiss();
                    if (mCurrent == StateMachine.State_Idle) {
                        AppHelper helper = AppHelper.getInstance();
                        AlertDialog.Builder builder = new AlertDialog.Builder(
                                helper.getContext());
                        builder.setTitle(R.string.app_name);
                        builder.setMessage(R.string.MSG_ServiceFail)
                                .setPositiveButton(R.string.STR_OK,
                                        new DialogInterface.OnClickListener() {
                                            public void onClick(
                                                    DialogInterface dialog,
                                                    int id) {
                                                System.exit(0);
                                            }
                                        });
                        builder.create();
                        builder.show();
                    }
                    // Log.i(GlobalDescribe.TAG, "onFinish");
                }

                @Override
                public void onTick(long millisUntilFinished) {
                    //Log.i(GlobalDescribe.TAG, "onTick");
                }
            }.start();
        }
    }

    @Override
    public void onRequestPermissionsResult(int permsRequestCode, String[] permissions, int[] grantResults) {
        try {
            Log.i("demo", "onRequestPermissionsResult(permsRequestCode=" + permsRequestCode + ")");
            AppPermissionHelper.onRequestPermissionsResult(this, permsRequestCode, permissions, grantResults);
        } catch (Exception e) {
            Log.i("demo", "onRequestPermissionsResult(permsRequestCode=" + permsRequestCode + ")", e);
        }
    }

    @Override
    protected void onDestroy() {

        if (helper != null)
            helper.Release();
        super.onDestroy();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {

        getMenuInflater().inflate(R.menu.menu_read, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            case R.id.action_reset:
                if (mPos == 0) {
                    ReadWriteFragment f1 = (ReadWriteFragment) adapter.getItem(0);
                    f1.ResetField();
                } else {
                    LockFragment f2 = (LockFragment) adapter.getItem(1);
                    f2.ResetField();
                }
                break;
            case R.id.action_about:
                View messageView = getLayoutInflater().inflate(R.layout.activity_about,
                        null, false);
                TextView AppVer = messageView.findViewById(R.id.txt_app_ver);
                TextView ServiceVer = messageView.findViewById(R.id.txt_service_ver);

                PackageManager manager = this.getPackageManager();
                try {
                    PackageInfo info = manager.getPackageInfo(
                            this.getPackageName(), 0);
                    AppVer.setText(info.versionName);

                } catch (PackageManager.NameNotFoundException e) {
                    e.printStackTrace();
                    AppVer.setText("0.0.0");
                }
                try {
                    PackageInfo info = manager.getPackageInfo(
                            "com.cipherlab.rfidservice", 0);
                    ServiceVer.setText(info.versionName);
                } catch (PackageManager.NameNotFoundException e) {
                    ServiceVer.setText("0.0.0");
                    e.printStackTrace();
                }

                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setIcon(R.mipmap.ic_launcher);
                builder.setTitle(R.string.app_name);
                builder.setView(messageView);
                builder.setPositiveButton(R.string.STR_OK,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface arg0, int arg1) {

                            }// end onClick()
                        });
                builder.create();
                builder.show();
                break;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onPause() {

        if (mCurrent != StateMachine.State_Idle) {
            if (helper.GetConnectionStatus()) {
                helper.SetHardwareTriggerStatus(true);
            }
        }

        try {
            unregisterReceiver(mMessageReceiver);
        } catch (IllegalArgumentException e) {

        }

        super.onPause();
    }

    @Override
    public void onResume() {

        if (mCurrent != StateMachine.State_Idle) {
            if (helper.GetConnectionStatus()) {
                helper.SetHardwareTriggerStatus(false);
            }
        }

        IntentFilter filter = new IntentFilter();
        filter.addAction(GeneralString.Intent_RFIDSERVICE_CONNECTED);
        filter.addAction(GeneralString.Intent_RFIDSERVICE_TAG_DATA);
        filter.addAction(GeneralString.Intent_GUN_Attached);
        filter.addAction(GeneralString.Intent_GUN_Unattached);
        registerReceiver(mMessageReceiver, filter);

        super.onResume();
    }

    private void setupViewPager(ViewPager viewPager) {
        adapter = new ViewPagerAdapter(getSupportFragmentManager());
        adapter.addFragment(new ReadWriteFragment(), getString(R.string.STR_AccessTitle1));
        adapter.addFragment(new LockFragment(), getString(R.string.STR_AccessTitle2));
        viewPager.setAdapter(adapter);
        viewPager.addOnPageChangeListener(myOnPageChangeListener);
    }

    private void EnableControl(boolean enable) {
        ReadWriteFragment f1 = (ReadWriteFragment) adapter.getItem(0);
        f1.EnableControl(enable);

        LockFragment f2 = (LockFragment) adapter.getItem(1);
        f2.EnableControl(enable);
    }

    private final OnPageChangeListener myOnPageChangeListener = new OnPageChangeListener() {

        @Override
        public void onPageScrollStateChanged(int state) {
            //Log.d("EZEdit", "onPageScrollStateChanged:" + state);
        }

        @Override
        public void onPageScrolled(int position,
                                   float positionOffset, int positionOffsetPixels) {
            //This method will be invoked when the current page is scrolled,
            //either as part of a programmatically initiated smooth scroll
            //or a user initiated touch scroll.
        }

        @Override
        public void onPageSelected(int position) {
            mPos = position;
            //Log.d("EZEdit", "onPageSelected:" + position);
            switch (mPos) {
                case 1:
                    ReadWriteFragment f1 = (ReadWriteFragment) adapter.getItem(0);
                    LockFragment f2 = (LockFragment) adapter.getItem(1);
                    f2.SetEPC(f1.GetEPC());
                    break;
            }
        }
    };

    private class ViewPagerAdapter extends FragmentPagerAdapter {
        private final List<Fragment> mFragmentList = new ArrayList<>();
        private final List<String> mFragmentTitleList = new ArrayList<>();

        public ViewPagerAdapter(FragmentManager manager) {
            super(manager);
        }

        @Override
        public Fragment getItem(int position) {
            return mFragmentList.get(position);
        }

        @Override
        public int getCount() {
            return mFragmentList.size();
        }

        public void addFragment(Fragment fragment, String title) {
            mFragmentList.add(fragment);
            mFragmentTitleList.add(title);
        }

        @Override
        public CharSequence getPageTitle(int position) {
            return mFragmentTitleList.get(position);
        }
    }

    private final BroadcastReceiver mMessageReceiver = new BroadcastReceiver() {

        @Override
        public void onReceive(Context context, Intent intent) {

            String action = intent.getAction();
            if (action == null)
                return;

            if (action.equals(GeneralString.Intent_RFIDSERVICE_CONNECTED)) {

                String name = intent.getStringExtra("PackageName");
                if (name == null) {
                    Log.i(AppHelper.TAG, "PackageName is null");
                    return;
                }

                if (!name.equals("sw.programme.rfid.uhf.demo")) {
                    Log.i(AppHelper.TAG, "PackageName is incorrect");
                    return;
                }
                mCurrent = StateMachine.State_Ready;
                if (PDialog != null)
                    PDialog.dismiss();
                Log.i(AppHelper.TAG, "Intent_RFIDSERVICE_CONNECTED");
                if (helper.GetConnectionStatus()) {
                    helper.SetHardwareTriggerStatus(false);
                }
                invalidateOptionsMenu();
            } else if (action.equals(GeneralString.Intent_RFIDSERVICE_TAG_DATA)) {
                switch (mPos) {
                    case 0: {
                        ReadWriteFragment f1 = (ReadWriteFragment) adapter.getItem(0);
                        f1.Process(context, intent);
                        break;
                    }
                    case 1: {
                        LockFragment f2 = (LockFragment) adapter.getItem(1);
                        f2.Process(context, intent);
                        break;
                    }
                }

            } else if (action.equals(GeneralString.Intent_GUN_Attached)) {
                if (helper.GetConnectionStatus()) {
                    helper.SetHardwareTriggerStatus(false);
                }
                EnableControl(true);
            } else if (action.equals(GeneralString.Intent_GUN_Unattached)) {

                EnableControl(false);
            }
        }
    };
}
