package sw.programme.rfid.uhf.demo;

import android.content.Context;
import android.content.Intent;
import android.content.res.Resources;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.InputFilter;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;

import com.cipherlab.rfid.GeneralString;
import com.cipherlab.rfid.RFIDMemoryBank;


public class ReadWriteFragment extends Fragment {

    private EditText mEPC;
    private EditText mPassword;
    private EditText mOffset;
    private EditText mLength;
    private EditText mData;
    private Spinner mBank;
    private Button mReadBtn;
    private Button mWriteBtn;

    private final AppHelper helper = AppHelper.getInstance();

    public ReadWriteFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        final View rootView = inflater.inflate(R.layout.fragment_access, container, false);

        mEPC = rootView.findViewById(R.id.edt_access_epc);
        mPassword = rootView.findViewById(R.id.edt_access_password);
        InputFilter[] FilterArray1 = new InputFilter[1];
        FilterArray1[0] = new InputFilter.LengthFilter(8);
        mPassword.setFilters(FilterArray1);

        mOffset = rootView.findViewById(R.id.edt_access_offset);
        InputFilter[] FilterArray2 = new InputFilter[1];
        FilterArray2[0] = new InputFilter.LengthFilter(3);
        mOffset.setFilters(FilterArray2);
        mLength = rootView.findViewById(R.id.edt_access_length);
        mLength.setFilters(FilterArray2);
        mData = rootView.findViewById(R.id.edt_access_data);

        Resources res = getResources();
        mBank = rootView.findViewById(R.id.select_access_bank);
        ArrayAdapter<String> adapter1 = new ArrayAdapter<String>(rootView.getContext(),
                R.layout.dropdown_item, res.getStringArray(R.array.bank_option));
        mBank.setAdapter(adapter1);
        mBank.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

            @Override
            public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id) {


            }

            @Override
            public void onNothingSelected(AdapterView<?> parentView) {

            }
        });

        mReadBtn = rootView.findViewById(R.id.btn_read);
        mReadBtn.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                mData.setText("");
                String epc_data = mEPC.getText().toString();
                if (StringEX.IsNullOrEmpty(epc_data)) {
                    helper.Inventory();
                } else {

                    byte[] EPCByteArray = GetEpcData();
                    if (EPCByteArray == null)
                        return;

                    byte[] PasswordByteArray = GetPassWordData();
                    if (PasswordByteArray == null)
                        return;

                    int arg1 = GetOffset();
                    if (arg1 < 0)
                        return;
                    int arg2 = GetLength();
                    if (arg2 < 0)
                        return;

                    switch (mBank.getSelectedItemPosition()) {
                        case 0:
                            helper.ReadTag(PasswordByteArray, EPCByteArray, RFIDMemoryBank.EPC, arg1, arg2);
                            break;
                        case 1:
                            helper.ReadTag(PasswordByteArray, EPCByteArray, RFIDMemoryBank.TID, arg1, arg2);
                            break;
                        case 2:
                            helper.ReadTag(PasswordByteArray, EPCByteArray, RFIDMemoryBank.Reserved, arg1, arg2);
                            break;
                        case 3:
                            helper.ReadTag(PasswordByteArray, EPCByteArray, RFIDMemoryBank.User, arg1, arg2);
                            break;
                    }
                }
            }
        });

        mWriteBtn = rootView.findViewById(R.id.btn_write);
        mWriteBtn.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                String epc_data = mEPC.getText().toString();

                if (StringEX.IsNullOrEmpty(epc_data)) {
                    StringEX.ShowMessage(getActivity(), R.string.MSG_EpcEmpty);
                    return;
                } else {

                    byte[] EPCByteArray = GetEpcData();
                    if (EPCByteArray == null)
                        return;

                    byte[] PasswordByteArray = GetPassWordData();
                    if (PasswordByteArray == null)
                        return;

                    byte[] DataByteArray = GetData();
                    if (DataByteArray == null)
                        return;

                    int arg1 = GetOffset();
                    if (arg1 < 0)
                        return;

                    switch (mBank.getSelectedItemPosition()) {
                        case 0:
                            helper.WriteTag(PasswordByteArray, EPCByteArray, RFIDMemoryBank.EPC, arg1, DataByteArray);
                            break;
                        case 1:
                            helper.WriteTag(PasswordByteArray, EPCByteArray, RFIDMemoryBank.TID, arg1, DataByteArray);
                            break;
                        case 2:
                            helper.WriteTag(PasswordByteArray, EPCByteArray, RFIDMemoryBank.Reserved, arg1, DataByteArray);
                            break;
                        case 3:
                            helper.WriteTag(PasswordByteArray, EPCByteArray, RFIDMemoryBank.User, arg1, DataByteArray);
                            break;
                    }
                }
            }
        });

        if (helper.GetConnectionStatus())
            EnableControl(true);
        else
            EnableControl(false);
        return rootView;
    }

    private byte[] GetEpcData() {
        String epc = mEPC.getText().toString();
        if (StringEX.IsNullOrEmpty(epc)) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_EpcEmpty);
            return null;
        }

        if (epc.length() % 2 == 1) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_BinaryError);
            return null;
        }

        byte[] ByteArray;
        try {
            Log.i(AppHelper.TAG, "EPC = " + epc);
            ByteArray = UtilityHelp.StringToByteArray(epc);
        } catch (Exception e) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_IncorrectEPC);
            return null;
        }

        return ByteArray;
    }

    private byte[] GetPassWordData() {
        String password = mPassword.getText().toString();
        byte[] PasswordByteArray;

        if (StringEX.IsNullOrEmpty(password)) {
            PasswordByteArray = new byte[]{(byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00};
        } else {
            if (password.length() != 8) {
                StringEX.ShowMessage(getActivity(), R.string.MSG_PasswordLengthError);
                return null;
            }

            try {
                Log.i(AppHelper.TAG, "Password = " + password);
                PasswordByteArray = UtilityHelp.StringToByteArray(password);
            } catch (Exception e) {
                StringEX.ShowMessage(getActivity(), R.string.MSG_IncorrectPassword);
                return null;
            }
        }
        return PasswordByteArray;
    }

    private byte[] GetData() {
        String data = mData.getText().toString();
        if (StringEX.IsNullOrEmpty(data)) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_DataEmpty);
            return null;
        }

        if (data.length() % 2 == 1) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_BinaryError);
            return null;
        }

        byte[] DataByteArray;
        try {
            DataByteArray = UtilityHelp.StringToByteArray(data);
        } catch (Exception e) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_IncorrectData);
            return null;
        }
        return DataByteArray;
    }

    private int GetOffset() {
        String offset = mOffset.getText().toString();
        if (StringEX.IsNullOrEmpty(offset)) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_StartEmpty);
            return -1;
        }
        int value = Integer.parseInt(offset);
        return (value * 2);
    }

    private int GetLength() {
        String length = mLength.getText().toString();
        if (StringEX.IsNullOrEmpty(length)) {
            StringEX.ShowMessage(getActivity(), R.string.MSG_LengthEmpty);
            return -1;
        }
        int value = Integer.parseInt(length);
        return (value * 2);
    }

    public void ResetField() {
        mEPC.setText("");
        mPassword.setText("");
        mOffset.setText("0");
        mLength.setText("0");
        mData.setText("");
        mBank.setSelection(0);
    }

    public void EnableControl(boolean flag) {
        if (mReadBtn != null)
            mReadBtn.setEnabled(flag);
        if (mWriteBtn != null)
            mWriteBtn.setEnabled(flag);
    }

    public String GetEPC() {
        return mEPC.getText().toString();
    }

    public void Process(Context context, Intent intent) {
        int type = intent.getIntExtra(GeneralString.EXTRA_DATA_TYPE, -1);
        Log.i(AppHelper.TAG, "type = " + type);
        int response = intent.getIntExtra(GeneralString.EXTRA_RESPONSE, -1);

        switch (response) {
            case 0: /* OPERATION_SUCCESS */
                Log.i(AppHelper.TAG, "OPERATION_SUCCESS");
                if (type == 1) {
                    String epc_data = mEPC.getText().toString();
                    String EPC = intent.getStringExtra(GeneralString.EXTRA_EPC);

                    Log.i(AppHelper.TAG, "EPC = " + EPC);
                    if (StringEX.IsNullOrEmpty(epc_data)) {
                        if (EPC != null)
                            mEPC.setText(EPC);
                    } else {
                        mData.setText(EPC);
                    }
                } else if (type == 3) {
                    String ReadData = intent.getStringExtra(GeneralString.EXTRA_ReadData);
                    Log.i(AppHelper.TAG, "ReadData = " + ReadData);
                    mData.setText(ReadData);
                }
                break;
            case 1: /* OPERATION_FINISH */
                Log.i(AppHelper.TAG, "OPERATION_FINISH");
                StringEX.ShowMessage(context, R.string.MSG_OperationSuccess);
                break;
            case 2: /* OPERATION_FAIL */
            case 7:
                Log.i(AppHelper.TAG, "OPERATION_FAIL");
                StringEX.ShowMessage(context, R.string.MSG_OperationFail);
                break;
            case 3: /* TAG_LOCK */
                Log.i(AppHelper.TAG, "TAG_LOCK");
                StringEX.ShowMessage(context, R.string.MSG_TagLocked);
                break;
            case 6:
                Log.i(AppHelper.TAG, "Access Password Fail");
                StringEX.ShowMessage(context, R.string.MSG_AccessPasswordFail);
                break;
            case 251: /* DEVICE_BUSY */
                Log.i(AppHelper.TAG, "DEVICE_BUSY");
                StringEX.ShowMessage(context, R.string.MSG_OperationFail);
                break;
            default:
                Log.i(AppHelper.TAG, "response = " + response);
                break;
        }
    }
}
