package sw.programme.rfid.uhf.permission;

/**
 * Created by miller.pan on 2016/11/29.
 */

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.provider.Settings;
import android.support.v4.app.ActivityCompat;
import android.util.Log;

import java.util.List;

import sw.programme.rfid.uhf.demo.MainActivity;


public class PermissionHelper {
    private static String TAG = "PermissionHelper";

    public static boolean shouldAskPermission() {
        return (Build.VERSION.SDK_INT > Build.VERSION_CODES.LOLLIPOP_MR1);
    }

    public static boolean checkSelfPermission(Context context, String permission) {
        try {
            int hasWriteFilePermission = ActivityCompat.checkSelfPermission(context, permission);
            if (hasWriteFilePermission == PackageManager.PERMISSION_GRANTED) {
                return true;
            }

            return false;
        } catch (Exception e) {
            Log.w(TAG, "checkSelfPermission(context=" + context + ",permission=" + permission + ")", e);
            return false;
        }
    }

    public static void requestPermissions(Activity activity, List<String> permissionList, int requestCode) {
        try {
            if (permissionList == null) {
                return;
            }

            int size = permissionList.size();
            String[] permissions = new String[size];
            for (int i = 0; i < size; i++) {
                permissions[i] = permissionList.get(i);
            }

            ActivityCompat.requestPermissions(activity, permissions, requestCode);
        } catch (Exception e) {
            Log.w(TAG, "requestPermissions(activity=" + activity + ",permissionList=" + permissionList + ",requestCode=" + requestCode + ")", e);
        }
    }

    public static boolean canWriteFile(Context context) {
        if (PermissionHelper.shouldAskPermission() == false) {
            return true;
        }

        if (context == null) {
            Log.d(TAG, "No context is on canWriteFile()");
            return false;
        }

        try {
            boolean bWriteFilePermission = PermissionHelper.checkSelfPermission(context, android.Manifest.permission.WRITE_EXTERNAL_STORAGE);
            if (bWriteFilePermission == false) {
                Log.i(TAG, "No permission:WRITE_EXTERNAL_STORAGE");
            }

            return bWriteFilePermission;
        } catch (Exception e1) {
            Log.w(TAG, "Cannot check Permission:WRITE_EXTERNAL_STORAGE", e1);
            return false;
        }
    }

    /*public static boolean canSystemAlertWindowAndSettings(Context context) {
        if (PermissionHelper.shouldAskPermission() == false) {
            return true;
        }

        if (context == null) {
            Log.d(TAG, "No context is on canSystemAlertWindowAndSettings()");
            return false;
        }

        boolean b = Settings.canDrawOverlays(context);
        if (b == false) {
            Log.i(TAG, "No permission:WRITE_EXTERNAL_STORAGE");
        }

        return b;
    }*/

    public static final int REQUEST_CODE_ACTION_MANAGE_OVERLAY_PERMISSION = 103;

    public static boolean requestAlertWindowPermission(Activity activity) {

        if (activity == null) {
            Log.d(TAG, "No activity is on requestAlertWindowPermission!!");
            return false;
        }

        //if (PermissionHelper.canSystemAlertWindowAndSettings(activity) == true) {
        //     return false;
        //}

        try {
            Intent intent = new Intent(Settings.ACTION_MANAGE_OVERLAY_PERMISSION);
            intent.setData(Uri.parse("package:" + activity.getPackageName()));
            activity.startActivityForResult(intent, REQUEST_CODE_ACTION_MANAGE_OVERLAY_PERMISSION);

            return true;
        } catch (Exception e) {
            Log.w(TAG, "Cannot requestAlertWindowPermission", e);
        }

        return false;
    }

    public static void onActivityResultAlertWindowPermission(Activity activity, int requestCode, int resultCode, Intent data) {
        if (requestCode != REQUEST_CODE_ACTION_MANAGE_OVERLAY_PERMISSION) {
            Log.d(TAG, "Not requestCode is on onActivityResultAlertWindowPermission(activity=" + activity + ",requestCode=" + requestCode + ")");
            return;
        }

        if (activity == null) {
            Log.d(TAG, "No activity is on onActivityResultAlertWindowPermission!!");
            return;
        }

//        if (Settings.canDrawOverlays(activity))
//        {
//            LogHelper.i(TAG, "onActivityResult granted");
//        }

        try {
            activity.finish();
            Intent newService = new Intent(activity.getApplicationContext(), MainActivity.class);
            activity.startActivity(newService);
        } catch (Exception e) {
            Log.w(TAG, "The ex is on onActivityResultAlertAppNotification(activity=" + activity + ",requestCode=" + requestCode + ")", e);
        }
    }
}